#!/usr/bin/perl

use strict;

package Wendy::Db;

require Exporter;

our @ISA         = qw( Exporter );

our @EXPORT      = qw( dbconnect
		       wdbconnect
		       wdbbegin
		       wdbcommit
		       wdbrollback
		       dbprepare
		       wdbprepare
		       dbdisconnect
		       wdbdisconnect
		       dbquote
		       dbgeterror
		       wdbgeterror
		       wdbdo
		       dbselectrow
		       seqnext );

our @EXPORT_OK   = qw( dbconnect
		       wdbconnect
		       __do_connect
		       wdbbegin
		       wdbcommit
		       wdbrollback
		       dbprepare
		       wdbprepare
		       dbdisconnect
		       wdbdisconnect
		       dbquote
		       wdbquote
		       dbgeterror
		       wdbgeterror
		       dbdo
		       wdbdo
		       dbselectrow
		       seqnext );

our %EXPORT_TAGS = ( default => [ ] );
our $VERSION     = 1.00;

use Wendy::Config ':dbauth';
use DBI;
use DBD::Pg;

my $rdbh = undef;
my $wdbh = undef;

my $one_dbh = 0;

sub dbconnect
{
	unless( $rdbh )
	{
		my $rdbhost = CONF_DBHOST;

		if( ref $rdbhost )
		{
rDhHOm41w4tWnBxp:
			foreach my $host ( @$rdbhost )
			{
				if( $rdbh = &__do_connect( $host ) )
				{
					last rDhHOm41w4tWnBxp;
				}
			}
		} else
		{
			$rdbh = &__do_connect( $rdbhost );
		}
	}

	return $rdbh;
}


sub wdbconnect
{
	unless( $wdbh )
	{
		my $wdbhost = CONF_WDBHOST;

		if( $wdbhost )
		{
			if( ref $wdbhost )
			{
				
			      gQUZ58SVNa3exkcY:
				foreach my $host ( @$wdbhost )
				{
					if( $wdbh = &__do_connect( $host ) )
					{
						last gQUZ58SVNa3exkcY;
					}
				}
			} else
			{
				$wdbh = &__do_connect( $wdbhost );
			}
		} else
		{
			$one_dbh = 1;
			$wdbh = $rdbh;
		}
	}

	return $wdbh;
}

sub __do_connect
{
	my $host = shift;

	my $dbh = DBI -> connect( 'dbi:Pg:dbname=' . CONF_DBNAME . ';host=' . $host . '',
			          CONF_DBUSER,
				  CONF_DBPASSWORD,
				  {
					  RaiseError => 0
				  } );
	return $dbh;
}

sub wdbbegin
{
	$wdbh -> begin_work() or return undef;
	return 1;
}

sub wdbcommit
{
	$wdbh -> commit() or return undef;
	return 1;
}

sub wdbrollback
{
	$wdbh -> rollback() or return undef;
	return 1;
}

sub dbprepare
{
	my $sql = shift;
	my $sth = $rdbh -> prepare( $sql ) or return undef;

	return $sth;
}

sub wdbprepare
{
	my $sql = shift;
	my $sth = $wdbh -> prepare( $sql ) or return undef;

	return $sth;
}

sub dbselectrow
{
	return $rdbh -> selectrow_hashref( shift );
}

sub dbdo
{
	my $sql = shift;
	$rdbh -> do( $sql ) or return undef;

	return 1;
}

sub wdbdo
{
	my $sql = shift;
	$wdbh -> do( $sql ) or return undef;

	return 1;
}

sub dbgeterror
{
	return $rdbh -> errstr();
}

sub wdbgeterror
{
	return $wdbh -> errstr();
}

sub dbquote
{
	my $val = shift;
	return $rdbh -> quote( $val );
}

sub wdbquote
{
	my $val = shift;
	return $wdbh -> quote( $val );
}

sub dbdisconnect
{
	$rdbh -> disconnect();
	$rdbh = undef;

	if( $one_dbh )
	{
		$wdbh = undef;
	}
}

sub wdbdisconnect
{
	unless( $one_dbh )
	{
		if( $wdbh )
		{
			$wdbh -> disconnect();
			$wdbh = undef;
		}
	}
}

sub seqnext
{
	my $sname = shift;

	my $sql = sprintf( "SELECT nextval(%s) AS id", &dbquote( $sname ) );
	my $sth = &wdbprepare( $sql );
	$sth -> execute();
	my $data = $sth -> fetchrow_hashref();
	my $rv = $data -> { "id" };
	$sth -> finish();

	return int( $rv );
}

1;
